(function () {
    'use strict';

    // Define the common module 
    // Contains services:
    //  - common
    //  - logger
    //  - spinner
    var commonModule = angular.module('common', []);

    // Must configure the common service and set its 
    // events via the commonConfigProvider
    commonModule.provider('commonConfig', function () {
        this.config = {
            // These are the properties we need to set
            //controllerActivateSuccessEvent: '',
            //spinnerToggleEvent: ''
        };

        this.$get = function () {
            return {
                config: this.config
            };
        };
    });

    commonModule.factory('common',
        ['$q', '$rootScope', '$timeout', '$resource', 'commonConfig', 'logger', common]);

    function common($q, $rootScope, $timeout, $resource, commonConfig, logger) {
        var throttles = {};
        var calSuccess = function (message) { alert("Appointment added successfully to calendar");};
        var calError = function (message) { alert("Sorry...The appointment could not be added to calendar"); };
        var gaCommonPlugin;

        var service = {
            // common angular dependencies
            $broadcast: $broadcast,
            $q: $q,
            $resource: $resource,
            $timeout: $timeout,
            // generic
            activateController: activateController,
            createSearchThrottle: createSearchThrottle,
            debouncedThrottle: debouncedThrottle,
            isNumber: isNumber,
            logger: logger, // for accessibility
            textContains: textContains,
            showAlert: showAlert,
            showConfirm: showConfirm,
            playBeep: playBeep,
            vibrateDevice: vibrateDevice,
            isPhoneGap: isPhoneGap,
            isAndroid: isAndroid,
            isIOS: isIOS,
            shareSocially: shareSocially,
            addEventToCalendar: addEventToCalendar,
            removeEventToCalendar: removeEventToCalendar,
            gaTrackPage: gaTrackPage,
            gaTrackEvent :gaTrackEvent
        };
        document.addEventListener("deviceready", onDeviceReady, false);
        return service;
        
        function onDeviceReady() {       
            gaCommonPlugin = window.plugins.gaPlugin;
            //gaPlugin.init(nativePluginResultHandler, nativePluginErrorHandler, "UA-33654876-5", 10);
            gaCommonPlugin.init(nativePluginResultHandler, nativePluginErrorHandler, "UA-33654876-5", 10);//production
            // gaPlugin.init(nativePluginResultHandler, nativePluginErrorHandler, "UA-49305907-1", 10);//susanta

            // gaPlugin = window.plugins.gaPlugin;
            // gaPlugin.init(nativePluginResultHandler, nativePluginErrorHandler, "UA-49305907-1", 10);
        }
        function activateController(promises, controllerId) {
            return $q.all(promises).then(function (eventArgs) {
                var data = { controllerId: controllerId };
                $broadcast(commonConfig.config.controllerActivateSuccessEvent, data);
            });
        }

        function $broadcast() {
            return $rootScope.$broadcast.apply($rootScope, arguments);
        }

        function nativePluginResultHandler(result) {
            //alert('nativePluginResultHandler - '+result);
            // console.log('nativePluginResultHandler: ' + result);

        }

        function nativePluginErrorHandler(error) {
            //alert('nativePluginErrorHandler - '+error);
            //console.log('nativePluginErrorHandler: ' + error);
        }

        function gaTrackPage(pageName) {
            // negative or positive
            if (window.cordova) {
                gaCommonPlugin.trackPage(nativePluginResultHandler, nativePluginErrorHandler, pageName);
            }
            
            return ;
        }

        function gaTrackEvent(eventcategory, eventAction , eventLabel) {
            // negative or positive
            if (window.cordova) {
                gaCommonPlugin.trackEvent(nativePluginResultHandler, nativePluginErrorHandler, eventcategory, eventAction, eventLabel, 1);
            }
            return ;
        }

        function createSearchThrottle(viewmodel, list, filteredList, filter, delay) {
            // custom delay or use default
            delay = +delay || 300;
            // if only vm and list parameters were passed, set others by naming convention 
            if (!filteredList) {
                // assuming list is named sessions,
                // filteredList is filteredSessions
                filteredList = 'filtered' + list[0].toUpperCase() + list.substr(1).toLowerCase(); // string
                // filter function is named sessionFilter
                filter = list + 'Filter'; // function in string form
            }

            // create the filtering function we will call from here
            var filterFn = function () {
                // translates to ...
                // vm.filteredSessions 
                //      = vm.sessions.filter(function(item( { returns vm.sessionFilter (item) } );
                viewmodel[filteredList] = viewmodel[list].filter(function (item) {
                    return viewmodel[filter](item);
                });
            };

            return (function () {
                // Wrapped in outer IFFE so we can use closure 
                // over filterInputTimeout which references the timeout
                var filterInputTimeout;

                // return what becomes the 'applyFilter' function in the controller
                return function (searchNow) {
                    if (filterInputTimeout) {
                        $timeout.cancel(filterInputTimeout);
                        filterInputTimeout = null;
                    }
                    if (searchNow || !delay) {
                        filterFn();
                    } else {
                        filterInputTimeout = $timeout(filterFn, delay);
                    }
                };
            })();
        }

        function debouncedThrottle(key, callback, delay, immediate) {
            var defaultDelay = 1000;
            delay = delay || defaultDelay;
            if (throttles[key]) {
                $timeout.cancel(throttles[key]);
                throttles[key] = undefined;
            }
            if (immediate) {
                callback();
            } else {
                throttles[key] = $timeout(callback, delay);
            }
        }

        function isNumber(val) {
            // negative or positive


            return /^[-]?\d+$/.test(val);
        }

        function textContains(text, searchText) {
            return text && -1 !== text.toLowerCase().indexOf(searchText.toLowerCase());
        }



        function showAlert(message, title, buttonName) {
            navigator.notification.alert(
            message, // message: Dialog message (String)
            alertDismissed, // alertCallback: Callback to invoke when alert dialog is dismissed. (Function)
            title, // title: Dialog title (String) (Optional, Default: "Alert")
            buttonName // Button name (String) (Optional, Default: "OK")
            );
        }
        function alertDismissed() {
            alert("How dare you to dismiss it?");
        }

        function showConfirm(message, title, buttonLabels) {
            alert(message);
            var st;
          navigator.notification.confirm(
            message, // message: Dialog message (String)
            onConfirm, // confirmCallback: - Callback to invoke with index of button pressed (1, 2 or 3). (Number)
            title, // title: Dialog title (String) (Optional, Default: "Confirm")
            buttonLabels // buttonLabels: Comma separated string with button labels (String) (Optional, Default: "OK,Cancel")
            );
        
            return st;
        }

        // process the confirmation dialog result
        function onConfirm(button) {
            st = button;
           
            alert('You selected button ' + button);

            
        }

        //Beep device for "beepTimes" times
        function playBeep(beepTimes) {
            navigator.notification.beep(beepTimes);
        }

        //vibrate device for "milliseconds" duration
        function vibrateDevice(milliseconds) {
            navigator.notification.vibrate(milliseconds);
        }
        function isPhoneGap() {
            var app = document.URL.indexOf('http://') === -1 && document.URL.indexOf('https://') === -1;
            if (app) {
                // PhoneGap application
                return true;
            } else {
                // Web page
                return false;
            }
        }
        function isAndroid() {
            return navigator.userAgent.toLowerCase().indexOf("android") > -1;
        }

        function isIOS() {
            return navigator.userAgent.match(/(iPad|iPhone|iPod)/i) != null;
        }
        function shareSocially(message) {
            window.plugins.socialsharing.available(function (isAvailable) {
             
                if (isAvailable) {
                    // use a local image from inside the www folder:
                    // window.plugins.socialsharing.share('Some text', 'Some subject', null, 'http://www.nu.nl');
                    // window.plugins.socialsharing.share('Some text');

                    // window.plugins.socialsharing.share('test', null, 'data:image/png;base64,R0lGODlhDAAMALMBAP8AAP///wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAUKAAEALAAAAAAMAAwAQAQZMMhJK7iY4p3nlZ8XgmNlnibXdVqolmhcRQA7', null, function(e){alert("success: " + e)}, function(e){alert("error: " + e)});
                    //window.plugins.socialsharing.share(null, null, 'https://www.google.nl/images/srpr/logo11w.png', null);
                    window.plugins.socialsharing.share(message);
                    // alternative usage:

                    // 1) a local image from anywhere else (if permitted):
                    // window.plugins.socialsharing.share('Some text', 'http://domain.com', '/Users/username/Library/Application Support/iPhone/6.1/Applications/25A1E7CF-079F-438D-823B-55C6F8CD2DC0/Documents/.nl.x-services.appname/pics/img.jpg');

                    // 2) an image from the internet:
                    // window.plugins.socialsharing.share('Some text', "Some subject', 'http://domain.com', 'http://domain.com/image.jpg');

                    // 3) text and link:
                    // window.plugins.socialsharing.share('Some text and a link', '', '', 'http://www.nu.nl');
                }
            });
        }
        function addEventToCalendar(title, where, notes, startDate, endDate) {         
            //window.plugins.calendar.createEvent(title, where, notes, startDate, endDate, calSuccess, calError);
            window.plugins.calendar.createEventInteractively(title, where, notes, startDate, endDate, calSuccess, calError);
        }
        function removeEventToCalendar() {
            window.plugins.calendar.deleteEvent(title, where, notes, startDate, endDate, calSuccess, calError);
        }
    }
})();