(function () {
    'use strict';
    var notifications;
    var serviceId = 'datacontext';

    angular.module('app').factory(serviceId,
        ['common', 'repositories', datacontext]);

    function datacontext(common, repositories) {
        var $q = common.$q;
        var repoNames = ['appointment', 'phr', 'vaccination', 'profile', 'registration', 'search',
            'chronicdisease', 'digitallocker', 'cacheservice'];

        var service = {
            getDoctorsInfoForFavDoc: getDoctorsInfoForFavDoc,
            getDoctorsInfoForRecoDocs: getDoctorsInfoForRecoDocs,
            getDoctorsInfoForRecoProcessing: getDoctorsInfoForRecoProcessing,
            getNotifications: getNotifications
        };

        init();
        return service;

        function init() {
            defineLazyLoadedRepos();
        }


        // Add ES5 property to datacontext for each named repo
        function defineLazyLoadedRepos() {
            repoNames.forEach(function (name) {
                Object.defineProperty(service, name, {
                    configurable: true, // will redefine this property once
                    get: function () {
                        // The 1st time the repo is request via this property, 
                        // we ask the repositories for it (which will inject it).
                        var repo = repositories.getRepo(name);
                        // Rewrite this property to always return this repo;
                        // no longer redefinable
                        Object.defineProperty(service, name, {
                            value: repo,
                            configurable: false,
                            enumerable: true
                        });
                        return repo;
                    }
                });
            });
        }

        //////Block for Favourite Doctors

        function getDoctorsInfoForFavDoc() {
            var doctors = [
                { doctorNameInfo: "Dr. Brahma Acharya", doctorSpecialization: "Orthopedics", doctorLocation: "Chandrsekharpur" },
                { doctorNameInfo: "Dr. Ashok Mahapatra", doctorSpecialization: "Cardiology", doctorLocation: "Mastercanteen" },
                { doctorNameInfo: "Dr. Mihir Dash", doctorSpecialization: "Neurology", doctorLocation: "OUAT" },
                { doctorNameInfo: "Dr. Rashmi Mishra", doctorSpecialization: "Dentist", doctorLocation: "Acharya Vihar" }
            ];
            return $q.when(doctors);
        }

        //////Block for Recommended Doctors

        function getDoctorsInfoForRecoDocs() {
            var doctors = [
                { doctorNameInfo: "Dr. Madhab Nanda Kar", doctorSpecialization: "Oncology Surgeon", doctorLocation: "Chandrsekharpur", addedToFav: "false" },
                { doctorNameInfo: "Dr. Pramod Mohanty", doctorSpecialization: "Pediatrics", doctorLocation: "Mastercanteen", addedToFav: "false" },
                { doctorNameInfo: "Dr. Arijit Mahapatra", doctorSpecialization: "Pediatrics", doctorLocation: "OUAT", addedToFav: "false" },
                { doctorNameInfo: "Dr. Rashmi Mishra", doctorSpecialization: "Dentist", doctorLocation: "Acharya Vihar", addedToFav: "true" }
            ];
            return $q.when(doctors);
        }

        //////Block for Recommend in Process Doctors

        function getDoctorsInfoForRecoProcessing() {
            var doctors = [
                { doctorNameInfo: "Dr. Sanatan Behera", doctorSpecialization: "Orthopedics", doctorLocation: "Chandrsekharpur" },
                { doctorNameInfo: "Dr. S.N. Mallik", doctorSpecialization: "Pediatrics", doctorLocation: "Mastercanteen" }
            ];
            return $q.when(doctors);
        }

        //////Block Started for Notifications
        function getNotifications() {
            notifications = [
                { notificationId: 1, notificationMessage: "Empowering healthcare through innovation", type: "alert" },
                { notificationId: 2, notificationMessage: "Empowering healthcare through innovation", type: "success" },
                { notificationId: 3, notificationMessage: "Empowering healthcare through innovation", type: "info" },
                { notificationId: 4, notificationMessage: "Empowering healthcare through innovation", type: "alert" },
                { notificationId: 5, notificationMessage: "Empowering healthcare through innovation", type: "warning" }
            ];
            return $q.when(notifications);
        }
    }
})();