﻿(function () {
    'use strict';

    // Factory name is handy for logging
    var serviceId = 'repository.profile';

    angular.module('app').factory(serviceId, ['repository.abstract', 'config', RepositoryProfile]);

    function RepositoryProfile(AbstractRepository, config) {

        // Define the functions and properties to reveal.
        function Ctor() {
            this.serviceId = serviceId;
            // Exposed data access functions
            this.getMyProfile = getMyProfile;
            this.getFamilyMembers = getFamilyMembers;
            this.updateProfile = updateProfile;
            this.updateUserIDOrPassword = updateUserIDOrPassword;
            this.updateFamilyDetails = updateFamilyDetails;
        }

        AbstractRepository.extend(Ctor);
        return Ctor;

        function getFamilyMembers() {
            var self = this;
            return self.get('api/Consumer/GetFamilyList', config.localStorageNames.familyMembers);
        };

        function getMyProfile() {
            var self = this;
            return self.get('api/Consumer/GetLoggedOnConsumerDetails', config.localStorageNames.loggedOnConsumerDetails);
        };

        function updateProfile(consumerBasicProfile) {
            var self = this;
            return self.saveNew('api/Consumer/UpdateConsumerBasicProfile', consumerBasicProfile);
        }

        function updateUserIDOrPassword(newDetails) {
            console.log(newDetails);
            var self = this;
            return self.saveNew('api/Consumer/UpdateUserIDOrPassword', newDetails);
        }

        function updateFamilyDetails(familyDetails, familyUpdate) {
            var self = this;
            if (familyUpdate) {
                var consumerFamily = {
                    ParentConsumerID: familyDetails.parentConsumerID,
                    ConsumerID: familyDetails.consumerID,
                    Name: familyDetails.name,
                    Relation: familyDetails.relation,
                    DateOfBirth: familyDetails.dateOfBirth,
                    Contact: familyDetails.contact,
                    Sex: familyDetails.sex
                };
                return self.saveNew('api/Consumer/UpdateFamilyMember', consumerFamily);
            }
            else {
                var consumerFamily = {
                    Name: familyDetails.name,
                    Relation: familyDetails.relation,
                    DateOfBirth: familyDetails.dateOfBirth,
                    Contact: familyDetails.contact,
                    Sex: familyDetails.sex
                };
                return self.saveNew('api/Consumer/RegisterFamilyMember', consumerFamily);
            }
            
        };
    }
})();  