﻿(function () {
    'use strict';

    // Factory name is handy for logging
    var serviceId = 'userService';

    // Define the factory on the module.
    // Inject the dependencies. 
    // Point to the factory definition function.
    // TODO: replace app with your module name
    angular.module('app').factory(serviceId, ['$q','datacontext', 'localStorageService', userService]);

    function userService($q, datacontext, localStorageService) {
        // Define the functions and properties to reveal.
        var userInformation = {
            isLogged: false,
            userName: '',
            contact: '',
            userImageSrc:'',
            getLoggedInUser: getLoggedInUser,
            logout:logout
        };

        return userInformation;

        //#region Internal Methods        
        function getLoggedInUser() {
            //This method returns back a promise which needs to be resolved by the calling function
            //First check the loggrd in user details in local storage
            var deferred = $q.defer();
            var self = this;
            var locallyStoredUser = angular.fromJson(localStorageService.get('user'));
            if (locallyStoredUser) {
                self.userName = locallyStoredUser.firstName;
                self.isLogged = true;
                self.contact = locallyStoredUser.contact;               
            }
            else {
                //If not there, check with the server
                datacontext.registration.getLoggedInUser().then(function (data) {
                    if (data != undefined) {
                        self.isLogged = true;
                        self.userName = data[0].firstName;
                        self.contact = data[0].contact;
                        //Add to local storage
                        localStorageService.add('user', angular.toJson(data[0]));
                    }
                    deferred.resolve();
                });
            }
            return deferred.promise;
            //If still not there, the user has signed out
        }

        function logout() {
            var self = this;
            //Remove the localstorage data
            localStorageService.remove('user');
            //Remove everything actually
            localStorageService.clearAll();
            self.isLogged = false;
            self.userName = '';
            self.contact = '';
            self.userImageSrc = '';
        }

        //#endregion
    }
})();